# version 20180227-2
import bpy, bmesh

def pieces_to_armature(delete_orig=False):
    sel = [o for o in bpy.data.objects if o.select and o.type == 'MESH']
    if not sel:
        return
    sc = bpy.context.scene
    go_back = sc.frame_current
    start_frame = sc.frame_start
    end_frame = sc.frame_end
    sc.frame_set(start_frame)
    num_objs = len(sel)
    bpy.ops.object.mode_set()
    bpy.ops.object.select_all(action='DESELECT')
    for i in range(num_objs):
        obj = sel[i]
        name = obj.name
        clone_mesh = obj.data.copy()
        bm = bmesh.new()
        bm.from_mesh(clone_mesh)
        bm.transform(obj.matrix_world)
        dl = bm.verts.layers.deform.verify()
        for v in bm.verts:
            v[dl].clear()
            v[dl][0] = 1.0
        bm.to_mesh(clone_mesh)
        bm.free()
        clone_obj = bpy.data.objects.new("untitled", clone_mesh)
        clone_obj.vertex_groups.new(name)
        sc.objects.link(clone_obj)
        clone_obj.select = True
    new_mesh = bpy.data.meshes.new("baked-mesh")
    new_mobj = bpy.data.objects.new("baked", new_mesh)
    sc.objects.link(new_mobj)
    new_mobj.select = True
    sc.objects.active = new_mobj
    bpy.ops.object.join()
    new_skel = bpy.data.armatures.new("baked-bones")
    new_aobj = bpy.data.objects.new("baked-skeleton", new_skel)
    sc.objects.link(new_aobj)
    sc.objects.active = new_aobj
    bpy.ops.object.mode_set(mode='EDIT')
    ebs = new_skel.edit_bones
    for o in sel:
        eb = ebs.new(o.name)
        eb.head = (0.0, 0.0, 0.0)
        eb.tail = (0.0, 1.0, 0.0)
        eb.matrix = o.matrix_world
    root = ebs.new("root")
    root.head = (0.0, 0.0, 0.0)
    root.tail = (0.0, 1.0, 0.0)
    for eb in ebs:
        if eb == root:
            continue
        eb.parent = root
    root_name = root.name # actual name may not be "root"
    bpy.ops.object.mode_set()
    new_mobj.modifiers.new(type='ARMATURE', name="skel").object = new_aobj
    pbs = [new_aobj.pose.bones[a.name] for a in sel]
    ims = [a.matrix_world.inverted() for a in sel]
    chs = []
    l = "location"
    q = "rotation_quaternion"
    new_aobj.pose.bones[root_name].keyframe_insert(q) # init action
    fcs = new_aobj.animation_data.action.fcurves
    for pb in pbs:
        pb.keyframe_insert(l)
        pb.keyframe_insert(q)
        ns = "pose.bones[\"" + pb.name + "\"]."
        ln = ns + l
        qn = ns + q
        t = (fcs.find(ln, 0).keyframe_points,
             fcs.find(ln, 1).keyframe_points,
             fcs.find(ln, 2).keyframe_points,
             fcs.find(qn, 0).keyframe_points,
             fcs.find(qn, 1).keyframe_points,
             fcs.find(qn, 2).keyframe_points,
             fcs.find(qn, 3).keyframe_points)
        chs.append(t)
    frame = start_frame
    while frame <= end_frame:
        sc.frame_set(frame)
        for (txc,tyc,tzc,qwc,qxc,qyc,qzc), im, ob in zip(chs, ims, sel):
            m = im * ob.matrix_world
            tx, ty, tz = m.to_translation()
            qw, qx, qy, qz = m.to_quaternion()
            txc.insert(frame, tx)
            tyc.insert(frame, ty)
            tzc.insert(frame, tz)
            qwc.insert(frame, qw)
            qxc.insert(frame, qx)
            qyc.insert(frame, qy)
            qzc.insert(frame, qz)
        frame += 1
    sc.frame_set(go_back)
    if delete_orig:
        for o in sel:
            bpy.data.objects.remove(o)

pieces_to_armature(delete_orig=True)
        